@@ -9,19 +9,18 @@
 from .utils import validate_variables, apply_model_changes, set_up_simulation, exec_xpp_simulation, get_results_of_sed_variables
 from biosimulators_utils.combine.exec import exec_sedml_docs_in_archive
 from biosimulators_utils.config import get_config, Config  # noqa: F401
-from biosimulators_utils.log.data_model import CombineArchiveLog, TaskLog  # noqa: F401
+from biosimulators_utils.log.data_model import CombineArchiveLog, TaskLog, StandardOutputErrorCapturerLevel  # noqa: F401
 from biosimulators_utils.model_lang.xpp.validation import validate_model
 from biosimulators_utils.report.data_model import ReportFormat, SedDocumentResults  # noqa: F401
 from biosimulators_utils.sedml import validation
 from biosimulators_utils.sedml.data_model import (Task, ModelLanguage, ModelAttributeChange,  # noqa: F401
                                                   SteadyStateSimulation, UniformTimeCourseSimulation,
                                                   Variable, Symbol)
-from biosimulators_utils.sedml.exec import exec_sed_doc
+from biosimulators_utils.sedml.exec import exec_sed_doc as base_exec_sed_doc
 from biosimulators_utils.utils.core import raise_errors_warnings
 from biosimulators_utils.viz.data_model import VizFormat  # noqa: F401
-import functools
 
-__all__ = ['exec_sedml_docs_in_combine_archive', 'exec_sed_task']
+__all__ = ['exec_sedml_docs_in_combine_archive', 'exec_sed_doc', 'exec_sed_task', 'preprocess_sed_task']
 
 
 def exec_sedml_docs_in_combine_archive(archive_filename, out_dir, config=None):
@@ -44,20 +43,65 @@ def exec_sedml_docs_in_combine_archive(archive_filename, out_dir, config=None):
             * :obj:`SedDocumentResults`: results
             * :obj:`CombineArchiveLog`: log
     """
-    sed_doc_executer = functools.partial(exec_sed_doc, exec_sed_task)
-    return exec_sedml_docs_in_archive(sed_doc_executer, archive_filename, out_dir,
+    return exec_sedml_docs_in_archive(exec_sed_doc, archive_filename, out_dir,
                                       apply_xml_model_changes=False,
                                       config=config)
 
 
-def exec_sed_task(task, variables, log=None, config=None):
+def exec_sed_doc(doc, working_dir, base_out_path, rel_out_path=None,
+                 apply_xml_model_changes=False,
+                 log=None, indent=0, pretty_print_modified_xml_models=False,
+                 log_level=StandardOutputErrorCapturerLevel.c, config=None):
+    """ Execute the tasks specified in a SED document and generate the specified outputs
+
+    Args:
+        doc (:obj:`SedDocument` or :obj:`str`): SED document or a path to SED-ML file which defines a SED document
+        working_dir (:obj:`str`): working directory of the SED document (path relative to which models are located)
+
+        base_out_path (:obj:`str`): path to store the outputs
+
+            * CSV: directory in which to save outputs to files
+              ``{base_out_path}/{rel_out_path}/{report.id}.csv``
+            * HDF5: directory in which to save a single HDF5 file (``{base_out_path}/reports.h5``),
+              with reports at keys ``{rel_out_path}/{report.id}`` within the HDF5 file
+
+        rel_out_path (:obj:`str`, optional): path relative to :obj:`base_out_path` to store the outputs
+        apply_xml_model_changes (:obj:`bool`, optional): if :obj:`True`, apply any model changes specified in the SED-ML file before
+            calling :obj:`task_executer`.
+        log (:obj:`SedDocumentLog`, optional): log of the document
+        indent (:obj:`int`, optional): degree to indent status messages
+        pretty_print_modified_xml_models (:obj:`bool`, optional): if :obj:`True`, pretty print modified XML models
+        log_level (:obj:`StandardOutputErrorCapturerLevel`, optional): level at which to log output
+        config (:obj:`Config`, optional): BioSimulators common configuration
+        simulator_config (:obj:`SimulatorConfig`, optional): tellurium configuration
+
+    Returns:
+        :obj:`tuple`:
+
+            * :obj:`ReportResults`: results of each report
+            * :obj:`SedDocumentLog`: log of the document
+    """
+    return base_exec_sed_doc(exec_sed_task, doc, working_dir, base_out_path,
+                             rel_out_path=rel_out_path,
+                             apply_xml_model_changes=apply_xml_model_changes,
+                             log=log,
+                             indent=indent,
+                             pretty_print_modified_xml_models=pretty_print_modified_xml_models,
+                             log_level=log_level,
+                             config=config)
+
+
+def exec_sed_task(task, variables, preprocessed_task=None, log=None, config=None):
     """ Execute a task and save its results
 
     Args:
-       task (:obj:`Task`): task
-       variables (:obj:`list` of :obj:`Variable`): variables that should be recorded
-       log (:obj:`TaskLog`, optional): log for the task
-       config (:obj:`Config`, optional): BioSimulators common configuration
+        task (:obj:`Task`): task
+        variables (:obj:`list` of :obj:`Variable`): variables that should be recorded
+        preprocessed_task (:obj:`dict`, optional): preprocessed information about the task, including possible
+            model changes and variables. This can be used to avoid repeatedly executing the same initialization
+            for repeated calls to this method.
+        log (:obj:`TaskLog`, optional): log for the task
+        config (:obj:`Config`, optional): BioSimulators common configuration
 
     Returns:
         :obj:`tuple`:
@@ -76,6 +120,52 @@ def exec_sed_task(task, variables, log=None, config=None):
     if config.LOG and not log:
         log = TaskLog()
 
+    if preprocessed_task is None:
+        preprocessed_task = preprocess_sed_task(task, variables, config=config)
+
+    # validate task
+    model = task.model
+    sim = task.simulation
+
+    # read model
+    xpp_sim = preprocessed_task['xpp_simulation']
+
+    # model changes
+    apply_model_changes(xpp_sim, model.changes)
+
+    # setup simulation
+    exec_kisao_id = preprocessed_task['algorithm_kisao_id']
+
+    # run simulation
+    raw_results = exec_xpp_simulation(model.source, xpp_sim)
+
+    # transform results
+    variable_results = get_results_of_sed_variables(sim, raw_results, variables)
+
+    # log action
+    if config.LOG:
+        log.algorithm = exec_kisao_id
+        log.simulator_details = xpp_sim['simulation_method']
+
+    ############################
+    # return the result of each variable and log
+    return variable_results, log
+
+
+def preprocess_sed_task(task, variables, config=None):
+    """ Preprocess a SED task, including its possible model changes and variables. This is useful for avoiding
+    repeatedly initializing tasks on repeated calls of :obj:`exec_sed_task`.
+
+    Args:
+        task (:obj:`Task`): task
+        variables (:obj:`list` of :obj:`Variable`): variables that should be recorded
+        config (:obj:`Config`, optional): BioSimulators common configuration
+
+    Returns:
+        :obj:`object`: preprocessed information about the task
+    """
+    config = config or get_config()
+
     # validate task
     model = task.model
     sim = task.simulation
@@ -105,23 +195,10 @@ def exec_sed_task(task, variables, log=None, config=None):
     # validate observables
     validate_variables(xpp_sim, variables)
 
-    # model changes
-    apply_model_changes(xpp_sim, model.changes)
-
     # setup simulation
     exec_kisao_id = set_up_simulation(sim, xpp_sim['simulation_method'], config=config)
 
-    # run simulation
-    raw_results = exec_xpp_simulation(model.source, xpp_sim)
-
-    # transform results
-    variable_results = get_results_of_sed_variables(sim, raw_results, variables)
-
-    # log action
-    if config.LOG:
-        log.algorithm = exec_kisao_id
-        log.simulator_details = xpp_sim['simulation_method']
-
-    ############################
-    # return the result of each variable and log
-    return variable_results, log
+    return {
+        'xpp_simulation': xpp_sim,
+        'algorithm_kisao_id': exec_kisao_id,
+    }
